#include <math.h>
#include "annalyser.h"

int generate_tone(float *samples, float freq, int len)
{
	int i;
	for (i=0; i<len; i++) {
		samples[i]=cos((double) 2*PI*freq*i);
	}
}


float dc_level(float *x, float length)
{
	int i;
	double r=0;

	for (i=0; i<length; i++)
	{
		r+=x[i]/length;
	}

	return((float) r);
}

float max_abs(float *x, int length)
{
	int i;
	float max,y=0;

	max=x[0];
	for (i=1; i<length; i++)
	{
		y=(x[i]>=0)?x[i]:-x[i];
		if (y>max) {
			max=y;
		}
	}

	return max;
}

void complex_base_band(float *x, int length, float freq, float phase, float *yi, float *yq)
{
	int i;

	for (i=0; i<length; i++)
	{
		yi[i]=x[i]*cos((double) 2*PI*freq*i+phase);
		yq[i]=-x[i]*sin((double) 2*PI*freq*i+phase);
	}
}


float get_gain(float dc_i, float dc_q)
{
	dc_i*=dc_i;
	dc_q*=dc_q;

	return(sqrtf(dc_i+dc_q));
}

float get_phase(float dc_i, float dc_q)
{
	float ph=0;

	if (dc_i==0)
	{
		if (dc_q>=0)
			ph=PI/2.0;
		else
			ph=-PI/2.0;
	}
	else if (dc_q==0)
	{
		if (dc_i>=0)
			ph=0.0;
		else
			ph=-PI;
	}
	else
	{
		ph=atanf(dc_q/dc_i);
		if (dc_i<0)
		{
			if (dc_q<0)
				ph-=PI;
			else
				ph+=PI;
		}
	}
	return(ph);
}

#ifdef IMPULSE
void canal_impuls(int *delta, int *impulse)
{
	int i;
	int mostra, result;

    for (i=0; i<IMPULSIONAL_LEN; i++)
    {
        /*Introduce a delta into the channel*/
        if ((i>=IMPULSE_POSITION)&&(i<=IMPULSE_POSITION+LENGTH_IMPULSE))
            mostra=32000;
        else
            mostra=0;

        delta[i]=mostra;
        result=codec_io(mostra);

	    impulse[i]=result;
    }
}
#endif


