/** ALOE headers
 */
#include <phal_sw_api.h>
#include <math.h>
#include <swapi_utils.h>

#include "typetools.h"

#include "noise.h"

#include "itf_types.h"
#include "inputs.h"
#include "outputs.h"
#include "stats.h"

int longtti,bitsxblock;
int configured;

#define WITH_NOISE

float lastsnr=0;

#ifndef WITH_NOISE
float cached[1000];
#endif

int process_input(int len)
{
    int i;
    float max;

    if (datatype_in==TYPE_BITSTREAM) {
        len*=8;

        bin2float(input_data,inputf,len);

        /* and modulate */
        for (i=0;i<len;i++) {
            inputf[i]=inputf[i]?1:-1;
        }
    /** if we are receiving samples, convert to float */
    } else {
    	len = typeNsamplesArray(datatype_in,len);

        type2float(input_data,inputf,len,datatype_in);

        /* scale to -1..1 */
		max=inputf[0];
		for (i=0;i<len;i++) {
			if (abs(inputf[i])>max)
				max=inputf[i];
		}

		if (max==0) max=1.0;

		for (i=0;i<len;i++) {
			inputf[i]/=max;
		}
    }

    if (snr!=lastsnr) {
        printf("changed snr to %.2f\n",snr);fflush(0);
        lastsnr=snr;
    }

#ifdef WITH_NOISE
    if (snr<100.0) {
        addnoise(snr,len,inputf);
    }
#else
    for (i=0;i<len;i++) {
        inputf[i]+=cached[i%1000];
    }
#endif

    for (i=0;i<len;i++) {
        inputf[i] *= gain*(datatype_in?max:1.0);
    }

    float2type(inputf,output_data,len,datatype);

    /* send throught output */
    SendItf(0,typeSizeArray(datatype,len));

    return 1;
}

/** Run function.
 * @return 1 if ok, 0 if error
 */
int RunCustom()
{
    return 1;
}

int InitCustom()
{
    #ifndef WITH_NOISE
    memset(cached,0,sizeof(float)*1000);
    addnoise(snr,1000,cached);
    #endif
    return 1;
}

void ConfigInterfaces(){}


