/** ALOE headers
 */
#include <phal_sw_api.h>
#include <math.h>
#include <swapi_utils.h>

#include "typetools.h"

#include "itf_types.h"
#include "inputs.h"
#include "outputs.h"
#include "stats.h"

#include "rcos.h"

#include "fdacoefs.h"

float taps[MAX_COEFFS];
int mem[MAX_COEFFS];

int norm;

int process_input(int len)
{
    int i;
    int nsamples;
    float g;

    nsamples=typeNsamplesArray(datatype,len);

    type2int(input_data,input_i,nsamples,datatype);
    
    g=(float) gain/norm;

    for (i=0;i<nsamples;i++) {
        output_i[i]=(int) (convii(filter,mem,Ntaps,input_i[i])*g);
    }

    int2type(output_i,output_data,nsamples,datatype);
    
    /* send throught output */
    SendItf(0,len);

    return 1;
}

/** Run function.
 * @return 1 if ok, 0 if error
 */
int RunCustom()
{
    return 1;
}

int InitCustom()
{
    int i;
    

    if (Ntaps>MAX_COEFFS) {
        xprintf("Error number of coefficients too large (%d>%d)\n",Ntaps,MAX_COEFFS);
        return 1;
    }

    memset(taps,0,sizeof(float)*Ntaps);

    /* compute filter coeficients */
    switch(FilterType) {
        /* rect */
        case 0:
            memset(taps,1,sizeof(float)*Ntaps);
            for (i=0;i<Ntaps;i++) {
                filter[i]=(int) (taps[i]*scale);
            }
            break;
        case 1:
            /* raised cosine */
            rcos(taps,Ntaps,alpha,rate);
            for (i=0;i<Ntaps;i++) {
                filter[i]=(int) (taps[i]*scale);
            }
            break;
        case 2:
            /* root raised cosine */
            rrcos(taps,Ntaps,alpha,rate);
            for (i=0;i<Ntaps;i++) {
                filter[i]=(int) (taps[i]*scale);
            }
            break;
        case 3:
            Ntaps=IFFL;
            memcpy(filter,IFF,Ntaps*sizeof(int));
            break;
        default:
            printf("Error unknown filter type %d\n",FilterType);
    }
    norm=0;
    for (i=0;i<Ntaps;i++) {
        norm+=filter[i]*filter[i];
    }
    memset(mem,0,sizeof(int)*Ntaps);
    return 1;
}


void ConfigInterfaces()
{
    
}



