/** ALOE headers
 */
#include <phal_sw_api.h>
#include <math.h>
#include <swapi_utils.h>

#include "typetools.h"

#include "itf_types.h"
#include "inputs.h"
#include "outputs.h"
#include "stats.h"

int configured;

struct interleaver_h cur_ctrl;

int input_i[INPUT_MAX_DATA*8],output_i[OUTPUT_MAX_DATA*8];

int process_input(int len)
{
    int i;
    char *inc;
    short *ins;
    int *ini;
    float *inf;

    if (!configured) {
        if (!rows) {
            rows = typeNsamplesArray(datatype,len)/cols;
        } else if (!cols) {
            cols = typeNsamplesArray(datatype,len)/rows;
        } else if (!rows && !cols) {
            Log("Interleaver: Error not configured %d rows %d cols\n",rows,cols);
            return 0;
        }
        cur_ctrl.input_length=typeNsamplesArray(datatype,len);
        cur_ctrl.output_length=rows*cols;
        cur_ctrl.permutation = NONE;
        cur_ctrl.rows = rows;
        cur_ctrl.cols = cols;
    }

    if (len<typeSizeArray(datatype,cur_ctrl.input_length)) {
        Log("Interleaver: rows=%d cols=%d expected len=%d len=%d\n",rows,cols,typeSizeArray(datatype,cur_ctrl.input_length),len);
        return 0;
    }

    type2int(input_data,input_i,cur_ctrl.input_length,datatype);
    
    /** add padding symbols */
    for (i=cur_ctrl.input_length;i<rows*cols;i++) {
        input_i[i]=(int) cur_ctrl.padding_symbol;
    }

    Interleaving(input_i,output_i,&cur_ctrl);

    int2type(output_i,output_data,cur_ctrl.input_length,datatype);
    
    /* send throught output */
    SendItf(0,typeSizeArray(datatype,cur_ctrl.output_length));

    return 1;
}


int process_control(int len) {
    configured=1;
    rows=ctrlpkt.rows;
    cols=ctrlpkt.cols;
    memcpy(&cur_ctrl,&ctrlpkt,sizeof(struct interleaver_h));
    Log("Interleaver: rows=%d, cols=%d\n",rows,cols);
}

/** Run function.
 * @return 1 if ok, 0 if error
 */
int RunCustom()
{
    return 1;
}

int InitCustom()
{
    configured=0;
    return 1;
}


void ConfigInterfaces()
{
    
}



