/** ALOE headers
 */
#include <phal_sw_api.h>
#include <swapi_utils.h>

#include "turbodecoder.h"

#include "typetools.h"

#include "itf_types.h"
#include "inputs.h"
#include "outputs.h"
#include "stats.h"


struct turbodecoderConf ccfg;

int configured;

int iterations;


int process_input(int len) {
    int l,i;
    int rpm,wpm;
    int nsamples;

    /* if working without control interface, set blocklenght to received packet length */
    if (!configured) {
        numblocks = 1;
        longblock = (len-TOTALTAIL)/RATE;
    }

    nsamples = typeNsamplesArray(datatype,len);

    if (nsamples<(RATE*longblock+TOTALTAIL)*numblocks) {
        Log("TurboDecoder: Invalid input length: longcb %d numcb %d len %d\n",longblock,numblocks,len);
        return 0;
    }

    type2int(input_data,(int*) samples,nsamples,datatype);

   /* configure decoder options */
    ccfg.Long_CodeBlock=longblock;
    ccfg.Turbo_Dt=dynamic_halt;
    ccfg.Turbo_iteracions=max_iterations;
    ccfg.haltMethod=dh_method;

    rpm=0;
    wpm=0;
    
    /* for each codeblock */
    iterations=0;
    for (l=0;l<numblocks;l++) {
        iterations=TurboDecoder(samples,&output_bits[wpm],&ccfg,&real_halt);
        rpm+=RATE*longblock+TOTALTAIL;
        wpm+=longblock;
    }

    char2bin(output_bits,output_data,wpm);

    /* send throught output */
    SendItf(0,typeSizeArray(TYPE_BITSTREAM,wpm));

    return 1;
}

int process_control(int len) {
    configured=1;
    longblock=ctrlpkt.long_block;
    numblocks=ctrlpkt.num_blocks;
    Log("TurboDecoder: longcb=%d numcb=%d\n",longblock,numblocks);
}

/** Run function.
 * @return 1 if ok, 0 if error
 */
int RunCustom()
{
    return 1;
}

int InitCustom()
{
	if (longblock && numblocks) {
		configured = 1;
	} else {
	    configured=0;
	}
    return 1;
}

void ConfigInterfaces(){}




