
#ifndef _CVITERBI_H
#define	_CVITERBI_H

#define KMAX 		20	/* Max constraint length */
#define TWOTOTHEMMAX 	400	/* 2^(K - 1) -- change as required */
#define PI 3.141592654		/* circumference of circle divided by diameter */

#define MSG_LEN 	100000l /* how many bits in each test message */
#define DOENC            	/* test with convolutional encoding/Viterbi decoding */
#undef  DONOENC          	/* test with no coding */
#define LOESN0 		0.0     /* minimum Es/No at which to test */
#define HIESN0 		3.5     /* maximum Es/No at which to test */
#define ESN0STEP 	0.5    	/* Es/No increment for test driver */

#define MAXINT 		0x7FFFFFFF

/*Values for modulation parameter. Indicate the number of bits per symbol*/
#define _BPSK 	1
#define _QPSK 	2
#define _16QAM 	4
#define _64QAM 	6
#define NO_MOD	7

/*Values for cconv parameter CC CODE RATE*/
#define CODERATE1R2	0.5		/*1/2*/
#define CODERATE3R4	0.75		/*3/4*/
#define CODERATE2R3	0.666666	/*2/3*/
#define CODERATE5R6	0.833333	/*5/6*/
/*Values for channelcoding*/
#define CC	0
#define	CTC	1
#define LDPC	2 
/*Values for CCpolinomial*/
#define DEFAULT 0
#define CUSTOM	1
/*Values for K*/
#define KWIMAX	7



//CONTROL VITERBI
////////////////////////////////////////////////////////////////////////////////
///////////////////////DATA CONTROL/////////////////////////////////////////////
/*Define the module control parameters*/
typedef struct		//Defined according the control out needs//
{
  int ninputdata;	//Number of input data bytes//  
  int noutputdata;	//Number of output data bytes//
  			//Num Output Bytes=(Num Input Bytes - tail)*(coderate)//
  			//noutputbytes=(ninputbytes-tail)*CCcoderate//

  int channelcoding;	//	CC: Convolutional coding
  			//	CTC: Convolutional Turbo code
			//	LDPC: Low Density Parity Check
			  				
  float CCcoderate; 	//CCONC: 	Convolutional code rate//	
  int tail;		//CCONV: 	Number of bytes of tail: 1 byte; 8 bits //
  int CCpolinomial;	//0:default; 1:custom 
  int g[2][KMAX];	//Polinomials definition. KMAX=20
  int K;		//constraint length
  float EbNo;		//VITERBI: Adaptative quantiser
  
}CData_viterbi;		//CONTROL DATA VITERBI DECODER
////////////////////////////////////////////////////////////////////////////////
///////////////////////EXECUTION CONTROL////////////////////////////////////////
/*Define the execution control parameters*/
#define NONACTIVE 	0
#define ACTIVE		1	/*Defines a module as active*/
#define TEST_FLOW	2	/*Test flows: Bypass data flow mode*/
#define TEST_PROCESS	3	/*Test Module Process*/	
typedef struct		//Defined according the control out needs
{
  int status;		//Define the status
  int timeslot;		//Define the timeslot to produce the change
}CExec_viterbi;		//CONTROL EXECUTION VITERBI DECODER
////////////////////////////////////////////////////////////////////////////////
///////////////////////CONTROL TYPE/////////////////////////////////////////////
typedef struct		//Defined according the control out needs
{
  CData_viterbi CData;		
  CExec_viterbi CExec;		 
}CViterbi;		//Control Convolutional Decoder functionality


#endif
