////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//  MODULE:      dersolomon                                                   //
//                                                                            //
//  FUNCTION:    Reed Solomon decoder                                         //
//                                                                            //
//  Description: Bit Level Control mode for Downlink WiMAX IEEE802.16e        //
//               rel 2005                                                     //
//                                                                            //
//                                                                            //
//                                                                            //
//June 2009                                                                   //
////////////////////////////////////////////////////////////////////////////////



/** ALOE headers
 */
#include <phal_sw_api.h>
#include <math.h>
#include <swapi_utils.h>

#include "itf_types.h"
#include "inputs.h"
#include "outputs.h"
#include "stats.h"

#include "utils.h"
#include "typetools.h"
#include "dersol_code.h"


/*GLOBAL VARIABLES*/
int data_rcved=0;	//Data received-->data_rcved=1
			//Data non received-->data_rcved=0


/*PREDEFINED FUNCTIONS*/
void Init_Controlfunction();
int datainflow_bypass(int len);


//////////////////////////////////////////////////////////////////////////////////////////
//SQUELETON CALLED FUNCTIONS//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
//CONTROL ////////////////////////////////////////////////////////////////////////CONTROL/
//////////////////////////////////////////////////////////////////////////////////////////
//INPUT///////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

/*Returns the length(in control in type words) of the expected control flow at each time-slot*/
//Funtion activated if control flow has been received*/
/*Check INPUT_MAX_CONTROL*/
int get_controlin_length()
{
int length=1;

	length=control_length();
    	return(length);
}

/*Process the received control flow according to the length returned by get_controlin_length()*/
/*"len" indicates the number of control words to be processed*/
int process_inputcontrol(int len)
{
	process_control_input(len);

return(1);	/*return 1 if OK; return 0 if error*/
}

//////////////////////////////////////////////////////////////////////////////////////////
//OUTPUT//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

/*Returns the length (in control out type words) of the generated control flow at each time-slot*/
/*Check OUTPUT_MAX_CONTROL*/
int get_controlout_length()
{
int length=1;

    return(length);
}

/*Generate the control word to be send according the length provided by get_controlout_length()*/
/*"len" indicates the number of control words to be generated*/
int generate_outputcontrol(int len)
{

/*return 1 if OK; return 0 if error*/
return(1);
}



//////////////////////////////////////////////////////////////////////////////////////////
//DATA/////////////////////////////////////////////////////////////////////////////DATA///
//////////////////////////////////////////////////////////////////////////////////////////
//INPUT///////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

/*Returns the length(in data in type words) of the expected data flow at each time-slot*/
//Funtion activated only if data flow has been received*/
/*Check INPUT_MAX_DATA*/
int get_datain_length()
{
int length=1;

	length=datain_length();
    	return(length);
}
/*Process the received data flow according to the length returned by get_datain_length()*/
/** Run function. return 1 if ok, 0 if error*/
/*"len" indicates the number of data words to be processed*/
int process_inputdata(int len)
{
	//printf("dersolomon: process_inputdata\n");
	data_rcved=1;	//The execution of this function implies that enough data
			//has been received

	//printf("dersolomon: process_inputdata\n");
	//Testing flows purposes
	if(control_dersolomon[0].CExec.status==TEST_FLOW)datainflow_bypass(len);
	//Testing module processing functionality
	if(control_dersolomon[0].CExec.status==TEST_PROCESS)test_rsolomon();
	//Normal rsolomon encoder execution
	if(control_dersolomon[0].CExec.status==ACTIVE)process_datainflow(len);
	return 1;	
}


//////////////////////////////////////////////////////////////////////////////////////////
//OUTPUT//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

/*Returns the length (in data out type words) of the generated data flow at each time-slot*/
int get_dataout_length()
{
int length;

	length=0;
	if(data_rcved==0)return(length);
	data_rcved=0;
	length=dataout_length();
    	return(length);
}


/*Generate the control word to be send according the length provided by get_controlout_length()*/
/** Run function. return 1 if ok, 0 if error*/
/*"len" indicates the number of data words to be generated*/
int generate_outputdata(int len)
{
	return 1;
}



int InitCustom()
{
    Init_Controlfunction();
    /* Initialization the ECC library */
    initialize_ecc ();
    return 1;
}

int RunCustom()
{
    return 1;
}

//////////////////////////////////////////////////////////////////////////////////////////
//END SQUELETON CALLED FUNCTIONS//////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
//######################################################################################//
//////////////////////////////////////////////////////////////////////////////////////////
//ESPECIFIC MODULE FUNCTIONS//////////////////////////////////////////////////////////////
//CONTROL////////////////////////////////////////////////////////////////////////CONTROL//
//////////////////////////////////////////////////////////////////////////////////////////
void Init_Controlfunction()
{
/* 	output_control[0].CEx.status=NONACTIVE;
	output_control[0].CWi.usernumber=0;
	output_control[0].CWi.uncodedbytes=0;		//VERIFY
	output_control[0].CWi.modula=_BPSK;	
	output_control[0].CWi.coderate=CODERATE1R2;		
	output_control[0].CWi.rsolomon=0;
	output_control[0].CWi.channelcoding=CC;
	output_control[0].CWi.CCcoderate=CODERATE1R2;	//CODE RATE
							//1/2=0.5; No puncturing
							//2/3=0.66;  3/4=0.75;  5/6=0.83 puncturing
	output_control[0].CWi.tail=0;			//1 Byte (8 bits)
	output_control[0].CWi.interl=0;	
*/	
}

int control_length()
{
	return(INPUT_MAX_CONTROL);
}

int process_control_input(int len)
{
	if(len==1){

		printf("O---->DERSOLOMON (config): TSLOT%d, STATUS=%d\n",\
			control_dersolomon[0].CExec.timeslot,\
			control_dersolomon[0].CExec.status);
		printf("O       NumInputbytes=%d, NumOutputbytes=%d\n",\
				control_dersolomon[0].CData.ninputbytes,\
				control_dersolomon[0].CData.noutputbytes); 
		printf("O       modulation=%d (O: NOMOD, 1: BPSK, 2:QPSK, 4:16QAM, 6:64QAM \n",\
				control_dersolomon[0].CData.modulation);
		printf("O       RS num parity bytes=%d\n",\
				control_dersolomon[0].CData.rsolomon_Pbytes); 		
	}
}

//////////////////////////////////////////////////////////////////////////////////////////
//DATA//////////////////////////////////////////////////////////////////////////////DATA//
//////////////////////////////////////////////////////////////////////////////////////////

/*Perfom a bypass input output without any data modification*/
int datainflow_bypass(int len)
{
int i;

	for(i=0; i<len; i++){
		output_data[i]=input_data[i];
		}
 	return(1);
}



#define DATA_FLOW_LENGTH	0

/*data_length(): Define the data flow length in each time-slot*/
int datain_length()
{
int length;

	if(control_dersolomon[0].CExec.status==NONACTIVE)length=0;
	if(control_dersolomon[0].CExec.status==ACTIVE)length=control_dersolomon[0].CData.ninputbytes;
	if(control_dersolomon[0].CExec.status==TEST_FLOW)length=control_dersolomon[0].CData.ninputbytes;
	if(control_dersolomon[0].CExec.status==TEST_PROCESS)length=control_dersolomon[0].CData.ninputbytes;

	return(length);
}

#define BSIZE 4096

int process_datainflow(int len)
{
int i, numofbits, j, NPARITYBYTES, DATASIZE;
int info_in[BSIZE];
int info_out[BSIZE];
unsigned char msg[K_LENGTH];
unsigned char codeword[N_LENGTH];
unsigned char msg0[K_LENGTH] = {0xD4, 0xBA, 0xA1, 0x12, 0xF2, 0x74, 0x96,\
				0x30, 0x27, 0xD4, 0x88, 0x9C, 0x96, 0xE3,\
				0xA9, 0x52, 0xB3, 0x15, 0xAB, 0xFD, 0x92,\
				0x53, 0x07, 0x32, 0xC0, 0x62, 0x48, 0xF0,\
				0x19, 0x22, 0xE0, 0x91, 0x62, 0x1A, 0xC1};
int erasures[16];
int nerasures = 0;
	
	if (control_dersolomon[0].CData.modulation!=_BPSK) {

		NPARITYBYTES=control_dersolomon[0].CData.rsolomon_Pbytes;
		DATASIZE=len-NPARITYBYTES;

 		////Add errors according the Reed Solomom capabilities////
	//	printf("DERSOLOMON: ERRORS--------------------------------------------------\n");
		for(i=NPARITYBYTES+1; i<NPARITYBYTES+1+NPARITYBYTES/2; i++)
							byte_err(0xFF,i, (unsigned char *)input_data);
	
	/*	printf("DERSOLOMON: DECODER--------------------------------------------------\n");
  	*/	/////Decode data into codeword, with NPARITYBYTES parity bytes ////
   		rs802_11e_decoder(input_data, DATASIZE, NPARITYBYTES);
		
		len=DATASIZE;	
		/////Move data to output buffer////
		for (i=0; i<(len); i++)output_data[i]=input_data[i];	

		}
	else{
		//BPSK MODE: NO RSOLOMON CODE
		for (i=0; i< len; i++)output_data[i]=input_data[i];
			
	}
	return(1);
}

int dataout_length()
{
int length;

	length=control_dersolomon[0].CData.noutputbytes;
	return(length);
}

int generate_dataflow(int len)
{
int i;

	return(1);
}


#define TESTLENGTH   35

int test_rsolomon(int len)
{
int i, numofbits;
int info_in[BSIZE];
int info_out[BSIZE];
unsigned char msg[K_LENGTH];
unsigned char codeword[N_LENGTH];
unsigned char tests[TESTLENGTH]={0x45, 0x29, 0xC4, 0x79, 0xAD, 0x0F, 0x55,\
					0x28, 0xAD, 0x87, 0xB5, 0x76, 0x1A, 0x9C,\
					0x80, 0x50, 0x45, 0x1B, 0x9F, 0xD9, 0x2A,\
					0x88, 0x95, 0xEB, 0xAE, 0xB5, 0x2E, 0x03,\
					0x4F, 0x09, 0x14, 0x69, 0x58, 0x0A, 0x5D};
	
/*	if (input_control[0].CWi.modula != _BPSK)
		{
		fillzero(input_data,msg,&len); //Shortened
		encode_data (msg, K_LENGTH, codeword);
		delredundant(codeword,input_control[0].CWi.rsolomon,len);
		//for (i=0; i<(flow_rx.cntlW.rsolomon+flow_rx.data.length); i++)
		//	output_data[i]=codeword[i];		
		//flow_tx.data.length=flow_rx.data.length+flow_rx.cntlW.rsolomon;
		
		}
	else
		{
		//BPSK: NO RSOLOMON. INFO_OUT=INFO_IN
		for (i=0; i< len; i++)output_data[i]=input_data[i];
		//ADD TAIL BYTE
		//flow_tx.data.length=flow_rx.data.length+1;
		}
*/
	return(1);
}



//////////////////////////////////////////////////////////////////////////////////////////
//END OF ESPECIFIC MODULE FUNCTIONS///////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////



void ConfigInterfaces() {}


