//////////////////////////////////////////////////////////////////////////////////////////
//                                                                                      //
//  FILE:        simbolmap_imp.c                                                     	//
//                                                                                      //
//                                                                                      //
//  Description: Simbol mapping 				         		//
//               									//
//                            								//
//							                                //
//  Input:	 control_flow_in; data_flow_in                                          //
//                                                                                      //
//  Output:      data_flow_out                                                          //
//                                                                                      //
//June 2009                                                                             //
//////////////////////////////////////////////////////////////////////////////////////////


/** ALOE headers
 */
#include <phal_sw_api.h>
#include <math.h>
#include <swapi_utils.h>

#include "itf_types.h"
#include "inputs.h"
#include "outputs.h"
#include "stats.h"

#include "typetools.h"
#include "symbolmap_code.h"



#define NUMBITSBYTE	8

/*GLOBAL VARIABLES*/
int data_rcved;		//Data received-->data_rcved=1
			//Data non received-->data_rcved=0

int bufferB[INPUT_MAX_DATA];


/*PREDEFINED FUNCTIONS*/
void Init_Controlfunction();
int datainflow_bypass(int len);


//////////////////////////////////////////////////////////////////////////////////////////
//SQUELETON CALLED FUNCTIONS//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
//CONTROL ////////////////////////////////////////////////////////////////////////CONTROL/
//////////////////////////////////////////////////////////////////////////////////////////
//INPUT///////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

/*Returns the length(in control in type words) of the expected control flow at each 	*/
/*time-slot										*/
/*Funtion activated if control flow has been received					*/					
/*Check INPUT_MAX_CONTROL								*/
int get_controlin_length(){
    	return(INPUT_MAX_CONTROL);
}

/*Process the received control flow according to the length returned by 		*/
/*get_controlin_length()								*/
/*"len" indicates the number of control words to be processed				*/
int process_inputcontrol(int len){
int i;

	if(len==1){
		printf("O---->SIMBOLMAPP (config): TSLOT%d, STATUS=%d\n",\
			control_simbolmap.CExec.timeslot,\
			control_simbolmap.CExec.status);
		printf("O       NumInputbytes=%d, NumOutputbytes=%d\n",\
				control_simbolmap.CData.ninputdata,\
				control_simbolmap.CData.noutputdata); 
		printf("O       modulation=%d (1: BPSK, 2: QPSK, 4:16QAM, 6:64QAM \n",\
				control_simbolmap.CData.modulation);
		
		
	}
return(1);	/*return 1 if OK; return 0 if error*/
}

//////////////////////////////////////////////////////////////////////////////////////////
//OUTPUT//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

/*Returns the length (in control out type words) of the generated control flow at each	*/
/*time-slot										*/
/*Check OUTPUT_MAX_CONTROL								*/
int get_controlout_length(){
int length=1;

    return(length);
}

/*Generate the control word to be send according the length provided by 		*/
/*get_controlout_length()								*/
/*"len" indicates the number of control words to be generated				*/
int generate_outputcontrol(int len){
/*return 1 if OK; return 0 if error*/
return(1);
}



//////////////////////////////////////////////////////////////////////////////////////////
//DATA/////////////////////////////////////////////////////////////////////////////DATA///
//////////////////////////////////////////////////////////////////////////////////////////
//INPUT///////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

/*Returns the length(in data in type words) of the expected data flow at each time-slot	*/
//Funtion activated only if data flow has been received					*/
/*Check INPUT_MAX_DATA									*/
int get_datain_length()
{
int length=0;

	if(control_simbolmap.CExec.status==NONACTIVE)length=0;
	if(control_simbolmap.CExec.status==ACTIVE)length=\
				control_simbolmap.CData.ninputdata;
	if(control_simbolmap.CExec.status==TEST_FLOW)length=\
				control_simbolmap.CData.ninputdata;
	if(control_simbolmap.CExec.status==TEST_PROCESS)length=\
				control_simbolmap.CData.ninputdata;

    	return(length);
}
/*Process the received data flow according to the length returned by get_datain_length()*/
/** Run function. return 1 if ok, 0 if error						*/
/*"len" indicates the number of data words to be processed				*/
int process_inputdata(int len)
{
	//printf("viterbi: process_inputdata\n");
	data_rcved=1;	//The execution of this function implies that enough data has 
			//been received
	
	//Testing flows purposes
	if(control_simbolmap.CExec.status==TEST_FLOW)datainflow_bypass(len);
	//Testing module processing functionality
	if(control_simbolmap.CExec.status==TEST_PROCESS)test_simbolmap();
	//Normal modules execution
	if(control_simbolmap.CExec.status==ACTIVE)process_datainflow(len);
	return 1;	
}


//////////////////////////////////////////////////////////////////////////////////////////
//OUTPUT//////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////

/*Returns the length (in data out type words) of the generated data flow at each	*/
/*time-slot										*/
int get_dataout_length()
{
int length;

	length=0;
	if(data_rcved==0)return(length);
	data_rcved=0;
	//Testing flows purposes
	if(control_simbolmap.CExec.status==TEST_FLOW)
		length=control_simbolmap.CData.ninputdata;	//Data send=Data received
	//Testing module processing functionality
	if(control_simbolmap.CExec.status==TEST_PROCESS)
		length=control_simbolmap.CData.ninputdata;					//No data send
	//Normal execution
	if(control_simbolmap.CExec.status==ACTIVE)	
		length=dataout_length();
    	return(length);
}


/*Generate the control word to be send according the length provided by 		*/
/*get_controlout_length()								*/
/** Run function. return 1 if ok, 0 if error						*/
/*"len" indicates the number of data words to be generated				*/
int generate_outputdata(int len)
{
	return 1;
}



int InitCustom()
{
    return 1;
}

int RunCustom()
{
    return 1;
}
void ConfigInterfaces()
{
}


//////////////////////////////////////////////////////////////////////////////////////////
//END SQUELETON CALLED FUNCTIONS//////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////
//######################################################################################//
//////////////////////////////////////////////////////////////////////////////////////////
//ESPECIFIC MODULE FUNCTIONS//////////////////////////////////////////////////////////////
//CONTROL////////////////////////////////////////////////////////////////////////CONTROL//
//////////////////////////////////////////////////////////////////////////////////////////
void Init_Controlfunction()
{
	control_simbolmap.CExec.status=NONACTIVE;
	control_simbolmap.CData.ninputdata=0;
	control_simbolmap.CData.noutputdata=0;
}

//////////////////////////////////////////////////////////////////////////////////////////
//DATA//////////////////////////////////////////////////////////////////////////////DATA//
//////////////////////////////////////////////////////////////////////////////////////////

/*Perfom a bypass input output without any data modification*/
int datainflow_bypass(int len)
{
int i;
int info_C[INPUT_MAX_DATA];

	printf("SIMBOLMAP: datainflow_bypass() NON VALID OPTION\n");
	return(1);
}



int process_datainflow(int length)
{
int i, dvalid;
unsigned char bufferCH[INPUT_MAX_DATA];
static int one=0;
static int count=0;

        if(length<=0)return(1);
 	//DONWLOAD DATA
	for(i=0; i<length; i++)bufferCH[i]=(char)input_data[i];
	
	//CHAR TO BITS	
	length=MSBfirstchar2bin (bufferCH, bufferB, length);
	
	//SYMBOL MAPPING
	symbolmap (bufferB, length,\
		(symbol *)output_data, control_simbolmap.CData.modulation);
	
	length=length/control_simbolmap.CData.modulation;
	
	return(1);

}

int dataout_length()
{
int length;

	length=control_simbolmap.CData.noutputdata;
	//printf("SYMBOLMAP.dataout_length(): length=%d\n", length);
	return(length);
}

int generate_dataflow(int len)
{
int i;
	return(1);
}



int test_simbolmap(int length)
{
    printf("SIMBOLMAP: test_simbolmap() NON VALID OPTION\n");
	return(1);
}



//////////////////////////////////////////////////////////////////////////////////////////
//END OF ESPECIFIC MODULE FUNCTIONS///////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////





