/*
 * sync_master.c
 *
 * Copyright (c) 2009 Ismael Gomez-Miguelez, UPC <ismael.gomez at tsc.upc.edu>,
 *                    Xavier Reves, UPC <xavier.reves at tsc.upc.edu>
 * All rights reserved.
 *
 *
 * This file is part of ALOE.
 *
 * ALOE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * ALOE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ALOE.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "phid.h"
#include "itf_utils.h"
#include "phal_hw_api.h"
#include "sync.h"

#define ever (;;)

#define TEST

/*#define NO_URGENT*/

/*#define DEB*/

int sync_run;
int testing = 0;
char line_str[128];

int fd_w, fd_r;
hwitf_t id_w, id_r;
int error;

#define BUFF_SZ 128

#define MAX_CLIENTS 10
struct client {
    int fd_w;
    int fd_r;
    hwitf_t id_r;
    hwitf_t id_w;
};

void answer_packet(hwitf_t id)
{
    int n1, n2;
    int error;
    struct syncdata sd;
    
    n1 = hwapi_itf_rcv(fd_r, &sd, sizeof (struct syncdata));
    if (!n1) {
      printf("received 0\n");
        return;
    }

    get_time(&sd.ref_time);
    n2 = hwapi_itf_snd(fd_w, &sd, sizeof (struct syncdata));
    if (n1<sizeof (struct syncdata)) {
        xprintf("SYNC_MASTER: Size (%d) error reading sync information\n", n1);
        error = 1;
    }
    if (n2<sizeof (struct syncdata)) {
        xprintf("SYNC_MASTER: Size (%d) error writing sync information\n", n2);
        error = 1;
    }
    if (sd.key != SYNC_KEY) {
        xprintf("SYNC_MASTER: Sync Key received (0x%x) is not correct\n", sd.key);
        error = 1;
    }
#ifdef DEB
    xprintf("SYNC_MASTER: Sended sync packet, rcv time %d:%d snd time %d:%d\n",sd.tx_time.tv_sec,sd.tx_time.tv_usec,sd.ref_time.tv_sec,sd.ref_time.tv_usec);
#endif
}

/***********
 *   MAIN  *
 ***********/

int main(int argc, char **argv)
{
    struct sync_register sync_reg;
    int n;
    hwitf_t remote_id;

    if (!hwapi_init()) {
        xprintf("SYNC_MASTER: Error initiating hwapi\n");
        exit(0);
    }

    if (!utils_create_attach_ext_id(SYNC_SLAVE_ITF, BUFF_SZ,
            &fd_w, FLOW_WRITE_ONLY, &id_w)) {
        xprintf("SYNC_MASTER: Error configuring slave sync output itf\n");
        exit(0);
    }
    if (!utils_create_attach_ext_id(SYNC_SLAVE_ITF, BUFF_SZ,
            &fd_r, FLOW_READ_ONLY, &id_r)) {
        xprintf("SYNC_MASTER: Error configuring slave sync output itf\n");
        exit(0);
    }

    xprintf("SYNC_MASTER: Init Ok\n");

    do {
        n = hwapi_itf_rcv(fd_r, (int*) & sync_reg, sizeof (struct sync_register));
        if (!n)
            hwapi_relinquish_daemon();
    } while (!n);
    if (n < 0) {
        xprintf("SYNC_MASTER: Error reading from itf\n");
        exit(0);
    }

    xprintf("SYNC_MASTER: Received connection packet from pe 0x%x remote itf 0x%x. Sending local itf 0x%x\n", sync_reg.pe_id, sync_reg.itf_id, id_r);

    remote_id = sync_reg.itf_id;
    sync_reg.itf_id = id_r;

    if (!error) {
        sync_reg.cmd = SYNC_REGISTER_OK;
    } else {
        sync_reg.cmd = SYNC_REGISTER_ERROR;
    }
    if (hwapi_itf_snd(fd_w, &sync_reg, sizeof (struct sync_register)) <= 0) {
        xprintf("SYNC_MASTER: Error writting to itf\n");
        exit(0);
    }

#ifndef NO_URGENT
    if (!hwapi_itf_seturgent_out(id_w, remote_id)) {
        xprintf("SYNC_MASTER: Error setting option to interface\n");
        error = 1;
    }
    if (!hwapi_itf_seturgent_in(id_r, answer_packet)) {
        xprintf("SYNC_MASTER: Error setting option to interface\n");
        error = 1;
    }
#endif


    error = 0;
    while (!error) {
        #ifdef NO_URGENT
        answer_packet(0);
        #endif
        hwapi_relinquish_daemon();
    }

    return 0;
}
